/*
 * Classe que tem o objetivo de executar validações gerais em
 * telas e requisições. São utilizados três elementos: eventAggregator,
 * interceptor (restEngine) e Decorator. O eventAggregator é executado
 * ao iniciar ou finalizar requisições; o interceptor é executado antes
 * de iniciar as requisições, fazendo validações antes de permitir que
 * as mesmas prossigam; e o Decorator é executado antes de abrir qualquer
 * container, alterando-o antes de ser aberto.
 */
var PreRequests = function (ContainerDecoratorProvider, eventAggregator, ZHPromise, InventarioRotativo_Service, ScreenService, restEngine, templateManager) {

	var containerBeingValidated = 0;
	var requestSent = false;
	var self = this;

	/*
	 * Função para indicar que um container está sendo validado, impedindo
	 * que o interceptor sobrescreva as validações feitas pelo Decorator.
	 * É utilizado um valor numérico para que, caso haja mais de um Decorator
	 * sendo utilizado, não haja conflito de condição de corrida.
	 */
	self.setContainerBeingValidated = function () {
		containerBeingValidated++;
	};

	/*
	 * Função utilizado no final de uma validação de Decorator, para que
	 * o interceptor possa voltar a validar as requisições quando a variável
	 * chegar ao valor 0 (quando não houver validação de qualquer Decorator).
	 */
	self.freeContainerBeingValidated = function () {
		containerBeingValidated--;
	};

	/*
	 * Função para verificar se há algum Decorator validando um
	 * container. Caso haja, o número na variável 'containerBeingValidated'
	 * deve ser maior que 0, retornando true.
	 */
	self.getContainerIsBeingValidated = function () {
		return containerBeingValidated > 0;
	};

	/*
	 * 	Função que executa a validação de inventário rotativo ao entrar
	 * 	em telas que estão parametrizadas para realizar tal validação.
	 * 	Se houver inventário rotativo pendente para a unidade, será exibida
	 * 	uma mensagem e haverá o redirecionamento para uma página em branco,
	 * 	necessária para impedir o acesso à tela acessada.
	 */
	self.validaInventarioRotativo = function (container) {
		if (container.validaEstoque === true) {
			self.setContainerBeingValidated();
			var deferred = ZHPromise.defer();
			var containerName = container.name;
			InventarioRotativo_Service.checkInventarioPendente().then(function(inventarioValido){
				var inveRotaContainer = (containerName === "ger03302_inventario_rotativo");
				if (!(inventarioValido || inveRotaContainer)) {
					container.widgets = [];
					container.events = [];
					ScreenService.showMessage("Operação bloqueada! Existe inventário pendente para a unidade.").then(function(){
						ScreenService.openWindow("est#ger03302_blank_window");
					});
				} else if (inveRotaContainer && inventarioValido) {
					container.widgets = [];
					ScreenService.showMessage(ScreenService.i18n("Não há inventário pendente para a unidade.")).then(function(){
						ScreenService.openWindow("est#ger03302_blank_window");
					});
				} else if (inveRotaContainer) {
					container.validaEstoque = false;
				}
				deferred.resolve({
					container: container,
					inventarioValido: inventarioValido
				});
			}.bind(this)).catch(function (error) {
				container.widgets = [];
				container.events = [];
				ScreenService.showMessage("Houve um erro ao tentar conectar com o servidor.").then(function(){
					ScreenService.openWindow("est#ger03302_blank_window");
				});
				deferred.reject(error);
			}).finally(function(){
				self.freeContainerBeingValidated();
			});
			return deferred;
		}
	};

	/*
	 * Função que envia nos headers de qualquer requisição parâmetros que
	 * indicam se a tela atual valida estoque e qual unidade será validada.
	 * Para isso, o container da tela deve conter as seguinter propriedades:
	 *
	 *   * filialOnRequest - String  - CDFILIAL utilizada no processo.
	 *   Opcional. Buscará a unidade logada como default.
	 *
	 *   * validaEstoque   - Boolean - Indica se a tela verifica estoque.
	 *   Opcional. Valor default é false, não validando inventário rotativo
	 *   automaticamente.
	 */
	eventAggregator.subscribe('EventOnRequestStart', function (eventData) {
		var url = eventData.requestConfig.url;
		// if para impedir de adicionar os headers caso seja uma requisição
		// para gerar relatórios
		if (url.indexOf('generateReport') === -1 && url.indexOf('generateXLS') === -1) {
			var container = templateManager.container;
			var filial = false;
			var validaEstoque = false;
			if (container) {
				filial = container.filialOnRequest || false;
				validaEstoque = (container.validaEstoque !== undefined) ? container.validaEstoque : false;
			}
			eventData.requestConfig.headers.Filial = filial;
			eventData.requestConfig.headers.ValidaEstoque = validaEstoque;
		}
	});

	/*
	 * Registrador do interceptador de requisições do Inventário Rotativo.
	 * Toda requisição pro backend proveniente de telas que validam estoque
	 * será validada antes de ser concretizada. Isso impede o funcionamento
	 * de telas caso haja inventário rotativo pendente, assim como nos
	 * sistemas Delphi.
	 */
	restEngine.registerInterceptor({
		method: "ALL",
		interceptor: function (data) {
			if (!(requestSent || self.getContainerIsBeingValidated())) {
				var container = templateManager.container;
				if (container && container.validaEstoque === true && container.name !== 'ger03302_inventario_rotativo') {
					var deferred = ZHPromise.defer();
					// Variável de controle, para que a requisição da validação
					// realizada aqui não seja verificada.
					requestSent = true;
					InventarioRotativo_Service.checkInventarioPendente().then(function (inventarioIsValid) {
						if (inventarioIsValid) {
							deferred.resolve();
						} else {
							ScreenService.showMessage("Operação bloqueada! Existe inventário pendente para a unidade.");
							deferred.reject();
						}
					}).catch(function () {
						// Caso haja algum erro, impede a requisição.
						deferred.reject();
					}).finally(function () {
						requestSent = false;
					});
					return deferred;
				}
			}
		}
	});

	/*
	 * Função que trabalha a recursividade do mecanismo de preenchimento do
	 * DescriptionField. Recebe uma widget e caso a mesma tenha widgets filhas,
	 * todos os fields do campo "select" dessas será verificado.
	 */
	self.verificaSelectWidgetsAndChilds = function (widget) {
		validaDescriptionFieldSelect(widget);
		if (widget.widgets && widget.widgets.length) {
			widget.widgets.forEach(function (childWidget) {
				self.verificaSelectWidgetsAndChilds(childWidget);
			});
		}
	};

	/*
	 * Função local auxiliar a "verificaSelectWidgetsAndChilds", responsavél por
	 * validar se os campos select de uma widget tem a propriedade description
	 * field preenchida. Caso algum dos fields não tenham a propriedade, a função
	 * 'preencheDescriptionFieldSelect' é executada.
	 */
	function validaDescriptionFieldSelect(widget) {
		if (widget.fields && widget.fields.length) {
			var description;
			widget.fields.forEach(function (field) {
				description = null;
				if (field.isVisible && field.fields && (field.template == "field/select.html" || field.template == "field/select-multiple.html")) {
					if (field.descriptionField) {
						field.fields.forEach(function (childField) {
							if (childField.name == field.descriptionField)
								description = true;
						});
						if (description === null) {
							preencheDescriptionFieldSelect(field);
						}
					} else {
						preencheDescriptionFieldSelect(field);
					}
				}
			});
		}
	}

	/*
	 * Função local auxiliar a "validaDescriptionFieldSelect",
	 * preenche a propriedade "descriptionField" de fields select
	 * com valores default. Ordem de preenchimento: mesmo nome do
	 * field ou primeiro field filho visível.
	 */
	function preencheDescriptionFieldSelect(field) {
		var description = null;
		description = field.fields.filter(function (childField) {
			return childField.isVisible !== false && childField.name == field.name && childField.showOnList;
		});
		field.descriptionField = description && description.length ? description[0].name : null;

		if (!description.length) {
			description = field.fields.filter(function (childField) {
				return childField.isVisible !== false && childField.showOnList;
			});
			field.descriptionField = description && description.length ? description[0].name : null;
		}
	}

	/*
	 * Registrador do Decorator do Inventário Rotativo. Valida toda abertura
	 * de telas, impedindo que aquelas que validam estoque sejam abertas
	 * caso haja inventário rotativo pendente, assim como nos sistemas Delphi.
	 */
	ContainerDecoratorProvider.registerProvider(new InventarioRotativoDecorator(ZHPromise, this));

	/*
	 * Registrador do Decorator de preenchimento default de Description Field.
	 * Valida toda abertura de telas, e aquelas que conterem fields select sem
	 * a propriedade de DescriptionField serão alteradas. O Decorator só é ativado
	 * caso a propriedade "useDecoratorInDescriptionFieldSelectInSelect" tenha valor true.
	 * Normalmente a propriedade é definida no enviroment.js.
	 */
	if (window.useDecoratorInDescriptionFieldSelect)
		ContainerDecoratorProvider.registerProvider(new DescriptionFieldSelectDecorator(ZHPromise, this));
};

Configuration(function (ContextRegister) {
	ContextRegister.register('PreRequests', PreRequests);
});